% GSS for discrete problem
% update p_k directely
function [myflag, iter, samples, Hbar, xbar] = GASS_dis(N, K, test_func, M, Hl,initlow,inithigh, D, rho, alpha, step, choice)
% usage example:
%   [F,I,S,H,X]=GASS_dis(1000,300,@flip_sphere,50,-2e6, -10, 0.5, 41, 0.1, 0.05, 0);
% 
% Input parameters:
% N: number of samples per interation
% K: number of iterations
% test_func: @+function name of the test, a function to be maximized
% M: dimension of the test function
% Hl: lower or higher bound of test_func, meaning that test_func-Hl or Hl-test_func >0
% initlow: lower bound of the range of Optimal Solution in each Dimension
% inithigh: higher bound of the range of Optimal Solution in each Dimension
% D: Number of Discrete Points in Each Dimension
% rho: Percentage of Elite Samples
% alpha: Initial Step Size
% step: Step Size Parameter
% choice: max(0) or min(1)
%
% Output parameters:
% myflag: has optimal solution or out of maximum iterations or no optimal
% solution or error
% iter: actual number of iterations
% samples: number of samples
% Hbar: optimal value
% xbar: candidate solution

st=initlow;  %start point
dist=(inithigh-initlow)/(D-1);
p_k = 1/D*ones(1,M*D);  
X_k_indx=multirnd1(p_k,M,N);
X_k=st+(X_k_indx-1)*dist;

stop=0;
k=1;
myflag = 1;
samples=zeros(K,1);
Hbar = zeros(K,1);
xbar = zeros(K,M);
Ntotal=0;


while (stop==0)&&(k<=K)
    gamma_k=alpha/k^step; % stepsize in SA
    Ntotal=Ntotal+N;
    samples(k) = Ntotal;
  
   if(choice == 0)
        G = feval(test_func,X_k)-Hl;
    else
        G = -feval(test_func,X_k)+Hl;
    end
    if G<0 % this is a test for whether the user has correctly specified the parameter Hl
        myflag = 0;
        if choice == 0
            w = msgbox('Lower bound is too big, and cannot gaurantee: objective_function - lower_bound >0','Cannot Find Optimal Solution');
        else
            w = msgbox('Higher bound is too small, and cannot gaurantee: higher_bound - objective_function - >0','Cannot Find Optimal Solution');
        end
        waitfor(w);
        break;
    end
    [Hbar(k), indx] = max(G);
    xbar(k,:) = X_k(indx,:);
    clear X_k;
    if isnan(Hbar(k))==1  
        Hbar(k)=Hbar(k-1);
        xbar(k,:) =xbar(k-1,:);
        break
    end

    if k>2
    if Hbar(k-1)>max(G)     
        Hbar(k)=Hbar(k-1);
        xbar(k,:) =xbar(k-1,:);
    end 
    end
    
    %--use shape function---take the elite samples---
    [G_sort, I_sort] = sort(G,'descend');   % sorted performances
    gm = G_sort(ceil(N*rho));
    %G_u=(G>=gm);
    G_u=(G>=gm).*(G);
    %r=((1e-5)+abs(0.1*Hbar(k))/log(1+k))^(-1);
    %r=1e-3;
    %r=1/max(abs(G));
    %G_u=(G>=gm).*exp(r*G);
    
    w_k=(G_u)/sum(G_u);

for j=1:M
    D1f_k(:,(j-1)*D+1:j*D)=(X_k_indx(:,j)*ones(1,D)==([1:D]'*ones(1,N))');
end
clear X_k_indx;     
for i=1:D*M
    if p_k(i)==0
        p_k_reci(i)=0;
    else
        p_k_reci(i)=1/p_k(i);
    end
end
    Df_k=(p_k_reci'*ones(1,N))'.*D1f_k; 
    grad_k=w_k'*Df_k;
    Hes_k_inv=zeros(D*M,D*M);
    for i=1:M
        Hes_k_inv((i-1)*D+1:i*D,(i-1)*D+1:i*D)=-(grad_k((i-1)*D+1:i*D)'*grad_k((i-1)*D+1:i*D)+1e-5.*eye(D))\diag(ones(1,D));
    end
    d_theta_k=-(Hes_k_inv*grad_k')';
    
    %d_theta_k=grad_k;
    
    %Hes_k=-(Df_k-ones(N,1)*mean(Df_k))'*(w_k*ones(1,D*M).*(Df_k-ones(N,1)*mean(Df_k)));
    %Hes_k=-grad_k'*grad_k;
    %d_theta_k=(-inv(Hes_k + 1e-10.*eye(D*M))*grad_k')';
    %d_theta_k=(-(Hes_k + 1e-10.*eye(D*M))\diag(ones(1,D*M))*grad_k')';
    %d_theta_k=(-Hes_k\diag(ones(1,D*M))*grad_k')';
    
    if norm(grad_k) < 1e-30
        myflag = 1;
        break;
    else
    % update the parameter using an SA iteration
    
    for j=1:M
        p_k((j-1)*D+1:j*D)=projsplx(p_k((j-1)*D+1:j*D)+gamma_k*d_theta_k((j-1)*D+1:j*D));
        %p_k_sum(j)=sum(p_k((j-1)*D+1:j*D));
    end
    %p_k
    %p_k_sum
   clear d_theta_k grad_k p_k_reci Hes_k_inv;
    %p_k_hat=zeros(1,D*M);    
%     for j=1:M
%         [p_max,indx_p]=max(d_theta_k((j-1)*D+1:j*D));
%         p_k_hat((j-1)*D+indx_p)=1;
%     end

%     for j=1:M
%         p_k_hat((j-1)*D+1:j*D)=d_theta_k((j-1)*D+1:j*D)/sum(d_theta_k((j-1)*D+1:j*D));
%     end
    
%     for j=1:M
%         p_k_hat((j-1)*D+1:j*D)=projsplx(p_k((j-1)*D+1:j*D)+5*gamma_k*d_theta_k((j-1)*D+1:j*D));
%     end

    %p_k=p_k+gamma_k*(p_k_hat-p_k);
      
    % generate new samples
%N=max(100,floor(k^0.502));
    
    X_k_indx=multirnd1(p_k,M,N);
    X_k=st+(X_k_indx-1)*dist;
    
    end  %end if
      
    k=k+1;    
    clear G G_u w_k grad_k D1f_k Df_k;
end %end while
iter = k;
if (k<=K) && (myflag == 1)
    myflag = 1;
elseif k>K && myflag == 1
    myflag = 2;
else
    myflag = 0;
end
if k > 2
    k = k-1;
else
    k = 2;
end
for l=k:K
samples(l) = Ntotal;
Hbar(l)=Hbar(k-1);
xbar(l,:) =xbar(k-1,:);
Ntotal=Ntotal+N;
end
if(choice == 0)
    Hbar=Hbar+Hl;
else
    Hbar=-Hbar+Hl;
end