% GSS with <= constrain on simplex using Dirchlet distribution
function [myflag, iter, samples, Hbar, xbar] = GSS1(N, K, test_func, M1, M2, Hl, rho, alpha, step, choice)
% usage example:
%   [F,I,S,H,X]=GSS(1000,300,@flip_sphere,10, 5, -2e6, 0.05, 1, 0.05, 0);
% 
% Input parameters:
% N: number of samples per interation
% K: number of iterations
% test_func: @+function name of the test, a function to be maximized
% M1: Number of Variables per Constraint
% M2: Number of Constrains
% Hl: lower or higher bound of test_func, meaning that test_func-Hl or Hl-test_func >0
% init: initial solution of every dimension lower bound
% rho: Percentage of Elite Samples
% alpha: Initial Step Size
% step: Step Size Parameter
% choice: max(0) or min(1)
%
% Output parameters:
% myflag: has optimal solution or out of maximum iterations or no optimal
% solution or error
% iter: actual number of iterations
% samples: number of samples
% Hbar: optimal value
% xbar: candidate solution


theta_k=ones(1,M1*(M2+1));

for m1=1:M1
XX_k(:,(M2+1)*(m1-1)+1:(M2+1)*m1)=dirrnd(theta_k((M2+1)*(m1-1)+1:(M2+1)*m1),N);
X_k(:,M2*(m1-1)+1:M2*m1)=XX_k(:,(M2+1)*(m1-1)+1:(M2+1)*m1-1);
end

stop=0;
k=1;
myflag = 1;
samples=zeros(K,1);
Hbar = zeros(K,1);
xbar = zeros(K,M1*M2);
Ntotal=0;

while (stop==0)&&(k<=K)
    %gamma_k=30/(k+1000)^0.501; % stepsize in SA  
    gamma_k=alpha/k^step;
   % N_k = N*k^(0.6); % exponent on k should be in (0.501, 1.002] from the choice of gamma_k
    
    Ntotal=Ntotal+N;
    samples(k) = Ntotal;
  
    if(choice == 0)
        G = feval(test_func,X_k)-Hl;
    else
        G = -feval(test_func,X_k)+Hl;
    end
    if G<0 % this is a test for whether the user has correctly specified the parameter Hl
        myflag = 0;
        if choice == 0
            w = msgbox('Lower bound is too big, and cannot gaurantee: objective_function - lower_bound >0','Cannot Find Optimal Solution');
        else
            w = msgbox('Higher bound is too small, and cannot gaurantee: higher_bound - objective_function - >0','Cannot Find Optimal Solution');
        end
        waitfor(w);
        break;  
    end
    
    [Hbar(k), indx] = max(G);
    xbar(k,:) = X_k(indx,:);

    if isnan(Hbar(k))==1  
        Hbar(k)=Hbar(k-1);
        xbar(k,:) =xbar(k-1,:);
        break
    end

    if k>2
    if Hbar(k-1)>max(G)     
        Hbar(k)=Hbar(k-1);
        xbar(k,:) =xbar(k-1,:);
    end 
    end

    %--use shape function---take the elite samples---
    [G_sort, I_sort] = sort(G,'descend');   % sorted performances
    gm = G_sort(ceil(N*rho));
    G_u=(G>gm).*(G+1e10);

    % esitmate gradient and hessian
    w_k=G_u/sum(G_u);
    CX_k=log(XX_k);
    grad_k=w_k'*CX_k-mean(CX_k);  
    %Hes_k=(CX_k-ones(N,1)*mean(CX_k))'*(w_k*ones(1,2*M).*(CX_k-ones(N,1)*mean(CX_k)))-cov(CX_k);
    Hes_k=-cov(CX_k);
    Hes_inv_k=inv(Hes_k + 1e-30.*eye(M1*(M2+1)));  % ensure positive-definiteness
    if norm(grad_k) < 1e-30
        myflag = 1;
        break;
    else
    % update the parameter using an SA iteration
    theta_k=theta_k-gamma_k*(Hes_inv_k*grad_k')';
    
    % generate new samples
    %X_k=dirrnd(theta_k,N);
    for m1=1:M1
    XX_k(:,(M2+1)*(m1-1)+1:(M2+1)*m1)=dirrnd(theta_k((M2+1)*(m1-1)+1:(M2+1)*m1),N);
    X_k(:,M2*(m1-1)+1:M2*m1)=XX_k(:,(M2+1)*(m1-1)+1:(M2+1)*m1-1);
    end
    end  %end if

    k=k+1;
    N=N*alpha;
    %N=ceil(max(1000,(3/(2*10^5)*1/(1/k^1.5-1/(k+1)^1.5))^2));
    clear G G_u w_k theta1_k theta2_k CX_k grad_k Hes_k Hes_inv_k;
end %end while

iter = k;
if (k<=K) && (myflag == 1)
    myflag = 1;
elseif k>K && myflag == 1
    myflag = 2;
else
    myflag = 0;
end
if k > 2
    k = k-1;
else
    k = 2;
end
for l=k:K
samples(l) = Ntotal;
Hbar(l)=Hbar(k-1);
xbar(l,:) =xbar(k-1,:);
Ntotal=Ntotal+N;
end
if(choice == 0)
    Hbar=Hbar+Hl;
else
    Hbar=-Hbar+Hl;
end