% simulated annealing
% maximization problem

function [myflag, iter, samples, Hbar, xbar] = SA_dist(N, K, test_func, M, Hl,initlow,inithigh, D, alpha, step, temp,choice)
% usage example:
%   [F,I,S,H,X]=SA_dist(1000,300,@flip_sphere,50,-2e6, -10, 0.5, 41, 0.1, 0.05, 5000, 0);
% 
% Input parameters:
% N: number of samples per interation
% K: number of iterations
% test_func: @+function name of the test, a function to be maximized
% M: dimension of the test function
% Hl: lower or higher bound of test_func, meaning that test_func-Hl or Hl-test_func >0
% initlow: lower bound of the range of Optimal Solution in each Dimension
% inithigh: higher bound of the range of Optimal Solution in each Dimension
% D: Number of Discrete Points in Each Dimension
% rho: Percentage of Elite Samples
% alpha: Initial Step Size
% step: Step Size Parameter
% temp: initial temperature
% choice: max(0) or min(1)
%
% Output parameters:
% myflag: has optimal solution or out of maximum iterations or no optimal
% solution or error
% iter: actual number of iterations
% samples: number of samples
% Hbar: optimal value
% xbar: candidate solution
st=initlow;  %start point
dist=(inithigh-initlow)/(D-1);
X_k_indx=unidrnd(D,N,M);
X_k=st+(X_k_indx-1)*dist;

T = temp;    % initial temperature
%gamma = 0.995;    % temperature reduction factor
Hbar(1) = -inf;
myflag = 1;
samples(1)=0;
Ntotal=0;

for i = 2:1:K
    gamma_k=alpha/i^step;
    %gamma_k=2*10^(1)/(i+1000)^0.501;
    Ntotal=Ntotal+N;
    samples(i) = Ntotal;
    % change temperature
    %T1 = T*gamma+1e-10;
    %T1=1*10^(2)/log(i+1);
    %T1=1e-5;
    %T1=10^(-15)+10^(-1)*abs(Hbar(i-1))/log(i+1);
    % evaluate weights
    if(choice == 0)
        G = feval(test_func,X_k)-Hl;
    else
        G = -feval(test_func,X_k)+Hl;
    end
    if G<0 % this is a test for whether the user has correctly specified the parameter Hl
        myflag = 0;
        if choice == 0
            w = msgbox('Lower bound is too big, and cannot gaurantee: objective_function - lower_bound >0','Cannot Find Optimal Solution');
        else
            w = msgbox('Higher bound is too small, and cannot gaurantee: higher_bound - objective_function - >0','Cannot Find Optimal Solution');
        end
        waitfor(w);
        break;
    end
    
    % estimate max H and x in this interation
    [Hbar(i), indx] = max(G);
    Hbar(i)=Hbar(i)-Hl;
    xbar(i,:) = X_k(indx,:);
    
    if i>2
    if Hbar(i-1)>max(G)     
        Hbar(i)=Hbar(i-1);
        xbar(i,:) =xbar(i-1,:);
    end 
    end
    
%     rho=0.05;
%     [G_sort, I_sort] = sort(G,'descend');   % sorted performances
%     gm = G_sort(ceil(N*rho));
%     G_u=(G>=gm).*(G-min(G));
    G_u=G-min(G);
    
    w_k=G_u/sum(G_u);
    d_k=gamma_k*var(G)^(-1)*(w_k'*G-mean(G));
    %d_k=gamma_k*(w_k'*G-mean(G));
    T1 = 1/(1+d_k*T)*T+1e-10;
    %T1 = 1/(1+5*10^(-2)*T)*T+1e-10;
    
    % MCMC move
    % generate Y
    ind=unidrnd(M);
    Y_d1=zeros(N,M);
    Y_d1(:,ind)=(unidrnd(2,N,1)*2-3)*dist;
    %Y_d1=(unidrnd(2,N,M)*2-3)*dist;
    
    Y_d2=st+(unidrnd(D,N,M)-1)*dist;
    U1 = rand(N,1);
    Y_d = Y_d1.*((U1 <= 0.99)*ones(1,M)) + Y_d2.*((U1>0.99)*ones(1,M));
    Y = X_k + Y_d;
    %Y = X + randn(N,M);
    % acceptance prob
    dG = -feval(test_func,Y)+feval(test_func,X_k);
    rho = min(exp(dG/T1),1);
    % accept & reject
    U = rand(N,1);
    X_k = Y.*((U <= rho)*ones(1,M)) + X_k.*((U>rho)*ones(1,M));
    T=T1;
    %fprintf('%d   %12.10f \n',i, Hbar(i));
end
%figure;
%semilogy([1:1:K], Hbar);
iter = i;
if (i<=K) && (myflag == 1)
    myflag = 1;
elseif i>K && myflag == 1
    myflag = 2;
else
    myflag = 0;
end
if i > 2
    i = i-1;
else
    i = 2;
end
for l=i:K
samples(l) = Ntotal;
Hbar(l)=Hbar(i-1);
xbar(l,:) =xbar(i-1,:);
Ntotal=Ntotal+N;
end
if(choice == 0)
    Hbar=Hbar+Hl;
else
    Hbar=-Hbar+Hl;
end